class DeviceConfiguration:
    def __init__(self):
        self.MaxPinIO = 27
        self.MaxPinAnalog = 10
        self.PWMPins = {1, 2, 3, 4, 5, 6, 7, 8, 11}
        self.InterruptPins = {1, 2, 3, 4, 5, 6, 7, 12}
        self.AnalogPins = {1, 2, 3, 4, 5, 6, 7, 8, 9, 17, 23}




from typing import List
from DUELink.SerialInterface import SerialInterface
from DUELink.Stream import StreamController

class SoundController:
    def __init__(self, transport:SerialInterface, stream:StreamController):
        self.transport = transport
        self.stream = stream

    def Beep(self, pin:int, frequency:int, duration:int)->bool:

        if pin < 0 or pin > self.transport.DeviceConfig.MaxPinIO:
            raise ValueError("Invalid pin")
        #if frequency < 0 or frequency > 10000:
            #raise ValueError("Frequency is within range[0,10000] Hz")
        # if duration < 0 or duration > 1000:
        #     raise ValueError("duration is within range[0,1000] millisecond")
        
        cmd = "beep({0}, {1}, {2})".format(pin, frequency, duration)
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success
    
    def MelodyPlay(self, pin: int, notes: List[float]) -> bool:

        if pin < 0 or pin not in self.transport.DeviceConfig.PWMPins:
            raise ValueError("Invalid pin")
        if not isinstance(notes, list) and all(isinstance(i, float) for i in notes):
            raise ValueError("Notes is not the correct datatype. Enter a list of floats for melody notes.")

        for note in notes:
            if note < 0 or note > 10000:
                raise ValueError("Note Frequency is within range[0,10000] Hz")
        
        #cmd = "MelodyP({0}, {{{1}}})".format(pin, ", ".join(map(str, notes)))
        # declare a9 array
        count = len(notes)
        # declare a9 array
        cmd = f"dim a9[{count}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        # write data to a9
        ret = self.stream.WriteFloats("a9",notes)

        # write a9 to dmx
        cmd = f"MelodyP({pin},a9)"
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()

        return ret.success

    def MelodyStop(self, pin: int)->bool:

        if pin < 0 or pin not in self.transport.DeviceConfig.PWMPins:
            raise ValueError("Invalid pin")
        
        cmd = "MelodyS({0})".format(pin)
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success
    
    def Wave(self, pin: int, buffer: bytes, offset: int, count: int, freq:int, delay_us:int ) -> bool:
        
        # declare a9 array
        count = len(buffer)
        # declare a9 array
        cmd = f"dim b9[{count}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        # write data to a9
        ret = self.stream.WriteBytes("b9",buffer)

        # write a9 
        cmd = f"Wave({pin},b9, {offset}, {count}, {freq}, {delay_us})"
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()

        return ret.success
    
    def Sweep(self, pin: int, freq_start: int, freq_end: int, vol_start: int, vol_end:int, delay:int ) -> bool:            

        # write a9
        cmd = f"Sweep({pin}, {freq_start}, {freq_end}, {vol_start}, {vol_end}, {delay})"
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()

        return ret.success
        
    
        



from enum import Enum
from DUELink.SerialInterface import SerialInterface
from DUELink.Stream import StreamController

class FileSystemController:   

    def __init__(self, transport:SerialInterface, stream:StreamController):
        self.transport = transport
        self.stream = stream

    
    def __ParseReturn(self)->int:
        ret = self.transport.ReadResponse()
        
        if (ret.success):
            try:
                value = int(ret.response)
                return value
            except:
                return -1
        return -1

    def Mount(self, type: int, cs: int, baud: int, max_handle: int)->int:            
        cmd = f"FsMnt({type}, {cs}, {baud}, {max_handle})"
        self.transport.WriteCommand(cmd)
        
        return self.__ParseReturn()

    def UnMount(self)->int:        
        self.transport.WriteCommand("FsUnMnt()")
        return self.__ParseReturn()
    
    def Format(self, type: int, cs: int, baud: int)->int:            
        cmd = f"FsFmt({type}, {cs}, {baud})"
        self.transport.WriteCommand(cmd)
        
        return self.__ParseReturn()
    
    def Open(self, path: str, mode: int)->int:            
        cmd = f"FsOpen(\"{path}\", {mode})"
        self.transport.WriteCommand(cmd)
        
        return self.__ParseReturn()
    
    def Close(self, handle: int)->int:            
        cmd = f"FsClose({handle})"
        self.transport.WriteCommand(cmd)
        
        return self.__ParseReturn()
    
    def Write(self, handle: int, data: bytes)->int:  
        count = len(data)          
        
        cmd = f"dim b9[{count}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        ret = self.stream.WriteBytes("b9", data)

        cmd = f"FsWrite({handle}, b9)"
        self.transport.WriteCommand(cmd)
        return self.__ParseReturn()
    
    def Read(self, handle: int, data: bytearray)->int:  
        count = len(data)          
        
        cmd = f"dim b9[{count}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()
        
        cmd = f"FsRead({handle}, b9)"
        self.transport.WriteCommand(cmd)    
        self.transport.ReadResponse()

        ret = self.stream.ReadBytes("b9",data)
        return ret
    
    def Sync(self, handle: int)->int:            
        cmd = f"FsSync({handle})"
        self.transport.WriteCommand(cmd)
        
        return self.__ParseReturn()
    
    def Seek(self, handle: int, offset: int)->int:            
        cmd = f"FsSeek({handle},{offset})"
        self.transport.WriteCommand(cmd)
        
        return self.__ParseReturn()
    
    def Tell(self, handle: int)->int:            
        cmd = f"FsTell({handle})"
        self.transport.WriteCommand(cmd)
        
        return self.__ParseReturn()
    
    def Delete(self, path: str)->int:            
        cmd = f"FsDel(\"{path}\")"
        self.transport.WriteCommand(cmd)
        
        return self.__ParseReturn()
    
    def Find(self, path: str)->int:            
        cmd = f"FsFind(\"{path}\")"
        self.transport.WriteCommand(cmd)
        
        return self.__ParseReturn()
    
    def Size(self, path: str)->int:            
        cmd = f"Fsfsz(\"{path}\")"
        self.transport.WriteCommand(cmd)
        
        return self.__ParseReturn()
    

    

    
    
        




       






import time
import serial
from datetime import datetime, timedelta
from DUELink.DeviceConfiguration import DeviceConfiguration
import re

class SerialInterface:
    CommandCompleteText = ">"
    DefaultBaudRate = 115200

    DeviceConfig : DeviceConfiguration

    def __init__(self, portName):        
        self.ReadTimeout = 3
        self.portName = portName
        self.echo = True 
    

    def Connect(self):
        self.portName = serial.Serial(self.portName, self.DefaultBaudRate, parity=serial.PARITY_NONE, bytesize=8, stopbits=serial.STOPBITS_ONE)
        self.portName.timeout = self.ReadTimeout        
        time.sleep(0.1)
        self.Synchronize()


    def Disconnect(self):
        try:
            self.portName.close()
        except:
            pass
        self.port = None

    def Synchronize(self):
        cmd = bytearray(1)
        cmd[0] = 10 
        self.WriteRawData(cmd, 0, 1)        
        time.sleep(0.4)

        # cmd[0] = 0x1B 
        # self.WriteRawData(cmd, 0, 1)        
        # time.sleep(0.4)

        self.WriteCommand("sel(1)")

        end = datetime.now() + timedelta(seconds=self.ReadTimeout)

        while datetime.now() < end and self.portName.in_waiting == 0:
            time.sleep(0.001) # wait 1ms for sure

        if  datetime.now() > end:
            raise Exception("Sync device failed.")
                
        self.portName.reset_input_buffer()
        self.portName.reset_output_buffer() 
        
        
    #def RemoveEchoRespone(self, response, cmd):
    #    if cmd in response:
    #        response = response[len(cmd):]
    #
    #    return response

    # def CheckResult(self, actual, expected):
    #     if actual != expected:
    #         raise Exception(f"Expected {expected}, got {actual}.")
    
    def DiscardInBuffer(self):
        self.portName.reset_input_buffer()

    def DiscardOutBuffer(self):
        self.portName.reset_output_buffer()

    def WriteCommand(self, cmd):
        self.DiscardInBuffer()
        self.DiscardOutBuffer()

        #command = cmd.lower()
        # these commands - statement can't use with println        
        #statement_list = ["print", "dim", "run"]
        #for statement in statement_list:
        #    i = command.index(statement)
        #    if i == 0:
        #        break


        #if (
        #    command.find('print') == 0 or 
        #    command.find('dim') == 0 or
        #    command.find('run') == 0 or
        #    command.find('list') == 0 or
        #    command.find('new') == 0 or
        #    command.find('echo') == 0 or
        #    command.find('sel') == 0 or
        #    command.find('version') == 0 or
        #    command.find('alias') == 0 or            
        #    command.find('sprintf') == 0 
        #):
        #    self.__WriteLine(cmd)
        #elif self.EnabledAsio == True:
        #    newcmd = f"println({cmd})"
        #    self.__WriteLine(newcmd)
        #else:
        #    self.__WriteLine(cmd)
        self.__WriteLine(cmd)

    def __WriteLine(self, string):
        string += "\n"
        # print(string)
        self.portName.write(bytes(string, 'utf-8'))

    def ReadResponse(self):
        str = ""
        end = datetime.now() + timedelta(seconds=self.ReadTimeout)

        resp = CmdRespone()

        responseValid = True
        dump = 0
        total_receviced = 0

        while datetime.now() < end or self.portName.in_waiting > 0:
            if self.portName.in_waiting > 0:
                data = self.portName.read(1)
                str += data.decode()
                
                total_receviced = total_receviced + 1
                
                if data.decode()[0] == '\n':
                    end_newline_expired = datetime.now() + timedelta(milliseconds=50) # 50ms timeout for \n

                    while (self.portName.in_waiting == 0 and datetime.now() < end_newline_expired ):
                        time.sleep(0)
                    
                    # next byte can be >, &, !, $
                    if (self.portName.in_waiting > 0):
                        dump = self.portName.read(1)

                        if (dump.decode()[0] == '>' or dump.decode()[0] == '!' or dump.decode()[0] == '$'):
                            #valid data      
                            time.sleep(0.001) # wait 1ms for sure

                            if (self.portName.in_waiting > 0):
                                responseValid = False
                        elif (dump.decode()[0] == '\r'): #there is case 0\r\n\r\n> if use println("btnup(0)") example, this is valid
                            if (self.portName.in_waiting == 0):
                                time.sleep(0.001) 

                            if (self.portName.in_waiting > 0):
                                dump = self.portName.read(1)

                                if (dump.decode()[0] == '\n'):
                                    if (self.portName.in_waiting > 0):
                                        dump = self.portName.read(1)
                                else:
                                    responseValid = False
                            else:
                                responseValid = False
                        else:
                            # bad data
                            # One cmd send suppose one response, there is no 1234\r\n5678.... this will consider invalid response
                            responseValid = False
                    
                    if responseValid == False:
                        d = 0

                        while d != '\n' and datetime.now() < end:
                            if (self.portName.in_waiting > 0):
                                dump = self.portName.read(1)
                                d = dump.decode()[0]
                            else:
                                time.sleep(0.001) 
                            
                            if d == '\n':
                                if (self.portName.in_waiting > 0): # still bad data, repeat clean up
                                    d = 0 #reset to repeat the condition while loop

                    if str == "" or len(str) < 2: #reponse valid has to be xxx\r\n or \r\n, mean idx >=2
                        responseValid = False
                    elif responseValid == True:
                        if str[len(str)-2] != '\r':
                            responseValid = False
                        else:
                            str = str.replace("\n", "")
                            str = str.replace("\r", "")

                    break
                elif data.decode()[0] == '>' or data.decode()[0] == '&':
                    if total_receviced == 1:
                        time.sleep(0.002)
                        if (self.portName.in_waiting == 0):
                            resp.success = True
                            resp.response = ""
                            return resp
                    
                end = datetime.now() + timedelta(seconds=self.ReadTimeout) #reset timeout after valid data 

                

        self.portName.reset_input_buffer()
        self.portName.reset_output_buffer()

        resp.success = (total_receviced > 1) and (responseValid == True)
        resp.response = str

        return resp
    
    def ReadResponseRaw(self):
        str = ""
        end = datetime.now() + timedelta(seconds=self.ReadTimeout)

        resp = CmdRespone()

        while datetime.now() < end:
            if self.portName.in_waiting > 0:
                data = self.portName.read()
                str += data.decode()
                end = datetime.now() + timedelta(seconds=self.ReadTimeout)

        self.portName.reset_input_buffer()
        self.portName.reset_output_buffer()

        if str != "":
            if len(str) >= 3:
                resp.response= str[0:len(str)-3]

            resp.success = True

        return resp

    TransferBlockSizeMax = 512
    TransferBlockDelay = 0.005

    def WriteRawData(self, buffer, offset, count):
        block = int(count / self.TransferBlockSizeMax)
        remain = int(count % self.TransferBlockSizeMax)

        idx = offset

        while block > 0:
            self.portName.write(buffer[idx:idx + self.TransferBlockSizeMax])
            idx += self.TransferBlockSizeMax
            block -= 1
            time.sleep(self.TransferBlockDelay)

        if remain > 0:
            self.portName.write(buffer[idx:idx + remain])

            #time.sleep(self.TransferBlockDelay)

    def ReadRawData(self, buffer, offset, count):
        end = datetime.now() + timedelta(seconds=self.ReadTimeout)


        countleft = count
        totalRead = 0

        
        #while end > datetime.now():
            #read = self.portName.readinto(buffer[offset + totalRead:offset + count])
            #totalRead += read

            #if read > 0:
            #    end = datetime.now() + timedelta(seconds=self.ReadTimeout)

            #if totalRead == count:
            #    break

        #return totalRead

        data = self.portName.read(count)

        if len(data) == 0:
            return 0 

        for i in range(offset,offset + count):
            buffer[i] = data[i-offset]


        return count
    
    def BytesToRead(self):
        return self.portName.in_waiting
    
    def ReadByte(self):
        data = self.portName.read(1)
        return data.decode()[0]

class CmdRespone:
    def __init__(self):
        self.response = ""
        self.success = False
    ######################################## OLD VERSION #####################################################
    # def CmdResponse(self):
    #     self.response = ""
    #     self.success = False
    #
    # def ReadResponse(self) -> CmdResponse:
    #     str = self.leftOver
    #     end = datetime.now() + timedelta(seconds=self.ReadTimeout)
    #
    #     resp = SerialInterface.CmdResponse
    #
    #     while end > datetime.now():
    #         data = self.portName.read()
    #
    #         str += data.decode('utf-8')
    #
    #         str = str.replace("\n", "")
    #
    #         idx1 = str.find(">")
    #         idx2 = str.find("&")
    #
    #         if idx1 == -1 and idx2 == -1:
    #             continue
    #
    #         idx = idx1 if idx1 != -1 else idx2
    #
    #         self.leftOver = str[idx + 1:]
    #         resp.success = True
    #         resp.response = str[:idx]
    #
    #         idx3 = str.find("!")
    #         if idx3 != -1 and "error" in resp.response:
    #             resp.success = False
    #
    #         return resp
    #
    #     self.leftOver = ""
    #     self.portName.reset_input_buffer()
    #     self.portName.reset_output_buffer()
    #
    #     resp.success = False
    #     resp.response = ""
    #
    #     return resp
    #
    # TransferBlockSizeMax = 256
    # TransferBlockDelay = 1
    #
    #
    #
    # def GetVersion(self):
    #     command = "version()"
    #     self.WriteCommand(command)
    #     version = self.ReadResponse()
    #     self.ReadCommandComplete()
    #     if version["success"]:
    #         if self.echo and command in version["response"]:
    #             # echo is on => need to turn off
    #             self.TurnEchoOff()
    #             self.portName.reset_input_buffer()
    #             self.portName.reset_output_buffer()
    #             version["response"] = version["response"][len(command):]
    #     return version["response"]
    #
    # def RemoveEchoRespone(self, response, cmd):
    #     if cmd in response:
    #         response = response[len(cmd):]
    #     return response
    #
    # def WriteCommand(self, command):
    #     self.WriteCommand(command)
    #
    # def WriteCommand(self, string):
    #     string += "\n"
    #     print(string)
    #     self.portName.write(bytes(string, 'utf-8'))
    #
    # def ReadCommandComplete(self):
    #     self.check_result(str(self.ReadResponse()), self.CommandCompleteText)


    ############################### This Function delay the process too much ###########################################
    # def check_result(self,actual: str, expected: str):
    #     if actual != expected:
    #         raise ValueError(f"Expected {expected}, got {actual}.")

    ####################################################################################################################


    # class CmdResponse:
    #     def __init__(self):
    #         self.response = ""
    #         self.success = False





from typing import Optional

import time
from DUELink.SerialInterface import SerialInterface
from DUELink.Stream import StreamController

class SpiController:
    def __init__(self, transport:SerialInterface, stream:StreamController):
        self.transport = transport
        self.stream = stream

    # def Write(self, dataWrite: bytes, offset: int = 0, length: Optional[int] = None, chipselect: int = -1) -> bool:
    #     if length is None:
    #         length = len(dataWrite)

    #     return self.WriteRead(dataWrite, offset, length, None, 0, 0, chipselect)

    def Configuration(self, mode, frequency)->bool:
        
        if not isinstance(mode, int) or mode not in {0,1,2,3}:
            raise ValueError("Invalid mode. Enter an integer between 0-3.")
        
        if not isinstance(frequency, int) or (not 200 <= frequency <= 24000):
            raise ValueError("Invalid frequency. Enter an integer between 200-24000.")
    
        cmd = f"spicfg({mode}, {frequency})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success
    
    def WriteByte(self, data: int)->int:
        
        if not isinstance(data, int) or (not 0 <= data <= 255):
            raise ValueError("Enter only one byte as an integer into the data parameter.")
    
        cmd = f"spiwr({data})"
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()
        if ret.success:            
            try:
                value = int(ret.response)
                return value
            except:
                pass

        return 0

    # def Read(self, dataRead: bytearray, offset: int = 0, length: Optional[int] = None, chipselect: int = -1) -> bool:
    #     if length is None:
    #         length = len(dataRead)

    #     return self.WriteRead(None, 0, 0, dataRead, offset, length, chipselect)

    def WriteRead(self, dataWrite: bytes, dataRead: bytearray) -> bool:
        countWrite = len(dataWrite)
        countRead = len(dataRead)
        written = 0
        read = 0
        

        # declare b9 to write
        if countWrite > 0:    
            cmd = f"dim b9[{countWrite}]"
            self.transport.WriteCommand(cmd)
            self.transport.ReadResponse()

        # declare b8 to read
        if countRead > 0:
            cmd = f"dim b8[{countRead}]"
            self.transport.WriteCommand(cmd)
            self.transport.ReadResponse()

        # write data to b9 by stream
        if countWrite > 0:    
            written = self.stream.WriteBytes("b9", dataWrite)

        # issue spi cmd
        if countWrite > 0 and countRead > 0:
            cmd = f"SpiWrs(b9, b8)"
        elif countWrite > 0:
            cmd = f"SpiWrs(b9, 0)"
        else:
            cmd = f"SpiWrs(0, b8)"

        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        # use stream to read data to b8
        if countRead > 0:
            read = self.stream.ReadBytes("b8", dataRead)

        # return true since we can't check status if Asio(1)
        return (written == countWrite) and (read == countRead)

        # while countWrite > 0 or countRead > 0:
        #     num = countRead

        #     if countWrite < countRead:
        #         num = countWrite

        #     if countWrite == 0 :
        #         num = countRead

        #     if countRead == 0 :
        #         num = countWrite

        #     if (num > self.transport.TransferBlockSizeMax) :
        #         num = self.transport.TransferBlockSizeMax

        #     if countWrite > 0:
        #         self.transport.WriteRawData(dataWrite, offsetWrite, num)
        #         offsetWrite += num
        #         countWrite -= num

        #     if countRead > 0:
        #         self.transport.ReadRawData(dataRead, offsetRead, num)
        #         offsetRead += num
        #         countRead -= num            

    
    
    
    # def Configuration(self,mode: int,  frequencyKHz: int)-> bool:
    #     if mode > 3 or mode < 0:
    #         raise ValueError("Mode must be in range 0...3.")
        
    #     if frequencyKHz < 200  or frequencyKHz > 20000:
    #         raise ValueError("FrequencyKHz must be in range 200KHz to 20MHz.")
        
    #     cmd = f"palette({mode},{frequencyKHz})"

    #     self.transport.WriteCommand(cmd)

    #     res = self.transport.ReadResponse()
    #     return res.success
    




from enum import Enum
from DUELink.SerialInterface import SerialInterface
from DUELink.Stream import StreamController

class OtpController:   

    def __init__(self, transport:SerialInterface, stream:StreamController):
        self.transport = transport
        self.stream = stream

    def Write(self, address: int, data: bytes)->bool:
        count = len(data)
        # declare b9 array
        cmd = f"dim b9[{count}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        # write data to b9
        ret = self.stream.WriteBytes("b9",data)

        # write b9 to dmx
        cmd = f"OtpW({address},b9)"
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()

        return ret.success

    def Read(self, address: int)->int:
        cmd = f"OtpR({address})"
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()

        if ret.success:            
            try:
                value = int(ret.response)
                return value
            except:
                pass

        return -1
        
        




       






from enum import Enum
from DUELink.SerialInterface import SerialInterface


class PulseController:   

    def __init__(self, transport:SerialInterface):
        self.transport = transport


    def Read(self, pin: int, charge_t: int, charge_s: int, timeout: int):
        cmd = "PulseIn({0}, {1}, {2}, {3})".format(pin, charge_t, charge_s, timeout)
        self.transport.WriteCommand(cmd)        

        ret = self.transport.ReadResponse()

        if ret.success:            
            try:
                value = int(ret.response)
                return value
            except:
                pass

        return 0

        
        




       






class HumiditySensorType():  
    def __init__(self):
        pass

    def __get_Dht11(self):
        return 1
    def __get_Dht12(self):
        return 2
    def __get_Dht21(self):
        return 3
    def __get_Dht22(self):
        return 4

    def __set_empty(self, value: int):
        return   

    DHT11 = property(__get_Dht11, __set_empty)  
    DHT12 = property(__get_Dht12, __set_empty)  
    DHT21 = property(__get_Dht21, __set_empty)  
    DHT22 = property(__get_Dht22, __set_empty)   

class HudimityController:
    def __init__(self, transport):
        self.transport = transport

    def Read(self, pin: int, sensortype: int) -> float:

        if pin < 0 or pin > self.transport.DeviceConfig.MaxPinIO:
            raise ValueError("Invalid pin. Enter a pin between 0-27.")
        
        cmd = f"humid({pin},{sensortype})"
        self.transport.WriteCommand(cmd)

        res = self.transport.ReadResponse()

        return float(res.response)
    
    



from enum import Enum
import time
import re

class SystemController:
    class ResetOption(Enum):
        SystemReset = 0
        Bootloader = 1


    def __init__(self, transport):
        self.transport = transport
        self.Version = ""         

    def Reset(self, option : int):
        cmd = "reset({0})".format(1 if option == 1 else 0)
        self.transport.WriteCommand(cmd)

        #Erase all send reset twice
        if (option == 1):
            self.transport.ReadResponse()
            self.transport.WriteCommand(cmd)

        # The device will reset in bootloader or system reset
        self.transport.Disconnect()

    def GetTickMicroseconds(self):
        cmd = "tickus()"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        if res.success:
            try:
                return int(res.response)
            except:
                pass
        return -1
    
    def GetTickMilliseconds(self):
        cmd = "tickms()"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        if res.success:
            try:
                return int(res.response)
            except:
                pass
        return -1
    
    # def GetVersion(self):
        # command = "version()"
        # self.transport.WriteCommand(command)

        # version = self.transport.ReadResponse()

        

        # match = re.match(r"^([\w\s]+).*?(v[\d\.].*)", version.response)


        # if version.success:
            # self.transport.TurnEchoOff()
            # self.transport.portName.reset_input_buffer()
            # self.transport.portName.reset_output_buffer()
            # version.response = version.response[len(command):]

        # version_firmware = match.group(2).split(":")[0]
        # prod_id = match.group(2).split(":")[1]
        # version_boot_loader = match.group(2).split(":")[2]


        # return version_firmware, prod_id, version_boot_loader
    
    def Info(self, code):
        cmd = f"info({code})"
        self.transport.WriteCommand(cmd)

        response = self.transport.ReadResponse()

        if response.success:            
            try:
                value = int(response.response)
                return value
            except:
                pass

        return 0
        
    def StatLed(self, highPeriod: int, lowPeriod: int, count: int) -> bool:
        cmd = f"statled({highPeriod},{lowPeriod},{count})"
        self.transport.WriteCommand(cmd)

        res = self.transport.ReadResponse()
        return res.success
    
    def LowPower(self, mode: int, pin:int)-> bool:
        cmd = f"lowpwr({mode},{pin})"
        self.transport.WriteCommand(cmd)

        # shutdown no response
        return True

    









from DUELink.SerialInterface import SerialInterface

class ServoController:
    def __init__(self, transport:SerialInterface):
        self.transport = transport

    def Set(self, pin, position)->bool:
        if pin < 0 or pin >= self.transport.DeviceConfig.MaxPinIO or pin not in self.transport.DeviceConfig.PWMPins:
            raise ValueError('Invalid pin. Enter a valid PWM pin.')
        if position < 0 or position > 180:
            raise ValueError('Position must be in the range 0..180')

        cmd = 'servost({}, {})'.format(pin, position)
        self.transport.WriteCommand(cmd)

        response = self.transport.ReadResponse()

        return response.success




from DUELink.SerialInterface import SerialInterface

class DigitalController:    

    def __init__(self, transport:SerialInterface):
        self.transport = transport

    def Read(self, pin, inputType: int) -> bool:

        if pin < 0 or pin > self.transport.DeviceConfig.MaxPinIO:
            raise ValueError("Invalid pin")

        if not isinstance(inputType, int) or inputType not in (0, 1, 2):
            raise ValueError("Invalid inputType. Enter an integer 0-2")    

        cmd = f"dread({pin},{inputType})"
        self.transport.WriteCommand(cmd)

        ret = self.transport.ReadResponse()

        if ret.success:            
            try:
                value = int(ret.response)
                return value == 1
            except:
                pass

        return False

    def Write(self, pin: int, value: bool) -> bool:

        if pin < 0 or pin > self.transport.DeviceConfig.MaxPinIO:
            raise ValueError("Invalid pin")
        
        cmd = f"dwrite({pin},{1 if value else 0})"
        self.transport.WriteCommand(cmd)

        ret = self.transport.ReadResponse()

        return ret.success



from DUELink.Graphics import GraphicsType

class GraphicsTypeController():  
    def __init__(self):
        pass

    def __get_I2C(self):
        return GraphicsType.I2C
    def __get_SPI(self):
        return GraphicsType.SPI
    def __get_Neo(self):
        return GraphicsType.Neo
    def __get_Matrix5x5(self):
        return GraphicsType.Matrix5x5
    
    def __set_empty(self, value: int):
        return   

    I2c = property(__get_I2C, __set_empty)  
    Spi = property(__get_SPI, __set_empty)  
    Neo = property(__get_Neo, __set_empty)  
    Matrix5x5 = property(__get_Matrix5x5, __set_empty)  
    



        



        
    



from DUELink.SerialInterface import SerialInterface

class DistanceSensorController:
    def __init__(self, transport:SerialInterface):
        self.transport = transport

    def Read(self, trigPin, echoPin)->float:
        cmd = f'dist({trigPin},{echoPin})'
        self.transport.WriteCommand(cmd)

        ret = self.transport.ReadResponse()

        if ret.success:
            try:
                return float(ret.response)
            except ValueError:
                pass

        return -1



class TemperatureSensorType():  
    def __init__(self):
        pass

    def __get_Cpu(self):
        return 0
    def __get_Dht11(self):
        return 1
    def __get_Dht12(self):
        return 1
    def __get_Dht21(self):
        return 3
    def __get_Dht22(self):
        return 4

    def __set_empty(self, value: int):
        return   

    CPU = property(__get_Cpu, __set_empty)
    DHT11 = property(__get_Dht11, __set_empty)  
    DHT12 = property(__get_Dht12, __set_empty)  
    DHT21 = property(__get_Dht21, __set_empty)  
    DHT22 = property(__get_Dht22, __set_empty)    

class TemperatureController:
    def __init__(self, transport):
        self.transport = transport

    def Read(self, pin: int, sensortype: int) -> float:
        cmd = f"temp({pin},{sensortype})"
        self.transport.WriteCommand(cmd)

        res = self.transport.ReadResponse()
        return float(res.response)
        



from enum import Enum
from DUELink.SerialInterface import SerialInterface
from DUELink.Stream import StreamController

class RtcController:   

    def __init__(self, transport:SerialInterface, stream:StreamController):
        self.transport = transport
        self.stream = stream

    def Write(self, rtc_timedate: bytes)->bool:
        count = len(rtc_timedate)
        # declare b9 array
        cmd = f"dim b9[{count}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        # write data to b9
        ret = self.stream.WriteBytes("b9",rtc_timedate)

        # write b9 to rtc
        self.transport.WriteCommand("RtcW(b9)")
        ret = self.transport.ReadResponse()

        return ret.success

    def Read(self, rtc_timedate: bytearray)->int:
        count = len(rtc_timedate)
        # declare b9 array
        cmd = f"dim b9[{count}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        cmd = f"RtcR(b9)"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        ret = self.stream.ReadBytes("b9",rtc_timedate)

        return ret
        
    def Alarm(self, rtc_timedate: bytes)->bool:
        count = len(rtc_timedate)
        # declare b9 array
        cmd = f"dim b9[{count}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        # write data to b9
        ret = self.stream.WriteBytes("b9",rtc_timedate)

        # write b9 to rtc
        self.transport.WriteCommand("RtcA(b9)")
        ret = self.transport.ReadResponse()

        return ret.success
        
        




       






import time
from DUELink.SerialInterface import SerialInterface

class EngineController:
    def __init__(self, transport : SerialInterface):
        self.transport = transport
        self.loadscript = ""    

    # run("version()/list") return version string, so need to return a string
    def Run(self) -> str:        
        self.transport.WriteCommand("run")
        
        res = self.transport.ReadResponse()

        return res.response
    
    def StopAll(self) -> str:        
        self.transport.DiscardInBuffer()
        self.transport.DiscardOutBuffer()
        
        data = bytearray(1)
        data[0] = 27
        self.transport.WriteRawData(data, 0, len(data))
        
        res = self.transport.ReadResponse()

        return res.response
    
    def Select(self, num)->bool:
        cmd = f"sel({num})"

        self.transport.WriteCommand(cmd)

        res = self.transport.ReadResponse()

        return res.success
    
    def Record(self, script,region) -> bool:
        if region == 0:
            self.transport.WriteCommand("new all")  
            res = self.transport.ReadResponse() 
            if res.success == False:
                return False
        elif region == 1:
            self.transport.WriteCommand("region(1)")  
            res = self.transport.ReadResponse() 
            if res.success == False:
                return False
            
            self.transport.WriteCommand("new")  
            res = self.transport.ReadResponse() 
            if res.success == False:
                return False
        else:
            return False
        
        cmd = "pgmbrst()"

        raw = script.encode('ASCII')

        data = bytearray(len(raw) + 1)

        data[len(raw)] = 0

        data[0:len(raw)] = raw        

        self.transport.WriteCommand(cmd)

        res = self.transport.ReadResponse()

        if (res.success == False) :
            return False
        
        self.transport.WriteRawData(data, 0, len(data))

        res = self.transport.ReadResponse()

        return res.success
            
    def Read(self) -> str:
        cmd = "list"

        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponseRaw()

        return res.response 

    def ExecuteCommand(self, cmd:str) -> float:
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()

        if ret.success:
            try:
                return float(ret.response)
            except:
                pass

        return 0
    
    def ExecuteCommandRaw(self, cmd:str):
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()

        return ret.response


    

       


from typing import Optional
from DUELink.SerialInterface import SerialInterface
from DUELink.Stream import StreamController

class I2cController:
    def __init__(self, transport:SerialInterface, stream:StreamController):
        self.transport = transport
        self.stream = stream
        self.baudrate = 400

    def Configuration(self, baudrate)->bool:

        if not isinstance(baudrate, int):
            raise ValueError("Enter an integer for the baudrate.")

        self.baudrate = baudrate

        cmd = f"i2ccfg({baudrate})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success

    def WriteRead(self, address: int, dataWrite: bytes, dataRead: bytearray) -> bool:
        countWrite = len(dataWrite)
        countRead = len(dataRead)
        

        # declare b9 to write    
        cmd = f"dim b9[{countWrite}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        # declare b8 to read
        cmd = f"dim b8[{countRead}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        # write data to b9 by stream
        self.stream.WriteBytes("b9", dataWrite)

        # issue i2cwr cmd
        cmd = f"i2cwr({address}, b9, b8)"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        # use stream to read data to b8
        self.stream.ReadBytes("b8", dataRead)

        # return true since we can't check status if Asio(1)
        return True



from enum import IntEnum
from DUELink.Stream import StreamController


class GraphicsType(IntEnum):    
    I2C = 1
    SPI = 2
    Neo = 3
    Matrix5x5 = 4


class GraphicsController:
    def __init__(self, transport,stream:StreamController):
        self.transport = transport
        self.stream = stream
    
    def Configuration(self, displayType, config, width, height, mode)->bool:

        #if not isinstance(config, list) or not all(isinstance(x, int) and 0 <= x <= 255 for x in config):
        #    raise ValueError("Enter a list with one number into the config with a valid code for a display.")
    
        #inputConfig = map(hex, config)
        
        #inputConfigArray = ",".join(inputConfig)
        
        #inputConfigArray = "{" + inputConfigArray + "}"

        #cmd = "MelodyP({0}, {{{1}}})".format(pin, ", ".join(map(str, notes)))
        # declare a9 array
        count = len(config)
        # declare a9 array
        cmd = f"dim a9[{count}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        # write data to a9
        ret = self.stream.WriteFloats("a9",config)
        
        cmd = f"gfxcfg({displayType}, a9, {width}, {height}, {mode})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success
        
    def Show(self)->bool:
        cmd = "show()"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success

    def Clear(self, color)->bool:
        cmd = f"clear({color})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success

    def Pixel(self, color, x, y)->bool:
        cmd = f"pixel({color},{x},{y})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success

    def Circle(self, color, x, y, radius)->bool:
        cmd = f"circle({color},{x},{y},{radius})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success

    def Rect(self, color, x, y, width, height)->bool:
        cmd = f"rect({color},{x},{y},{width},{height})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success

    def Fill(self, color, x, y, width, height)->bool:
        cmd = f"fill({color},{x},{y},{width},{height})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success

    def Line(self, color, x1, y1, x2, y2)->bool:
        cmd = f"line({color},{x1},{y1},{x2},{y2})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success

    def Text(self, text, color, x, y)->bool:
        cmd = f"text(\"{text}\",{color},{x},{y})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success
    
    def TextT(self, text, color, x, y)->bool:
        cmd = f"textt(\"{text}\",{color},{x},{y})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success

    def TextS(self, text, color, x, y, scalewidth, scaleheight)->bool:
        cmd = f"texts(\"{text}\",{color},{x},{y},{scalewidth},{scaleheight})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success

    # def __Stream(self, data, color_depth: int):
    #     cmd = f"stream({color_depth})"
    #     self.transport.WriteCommand(cmd)
    #     res = self.transport.ReadResponse()

    #     if res.success:
    #         self.transport.WriteRawData(data, 0, len(data))
    #         # time.sleep(10)
    #         res = self.transport.ReadResponse()

    #     return res.success

    def DrawImageScale(self, img, x: int, y: int, width: int, height: int, transform: int, scaleWidth: int, scaleHeight: int) -> bool:

        if width <= 0 or height <= 0 or len(img) < width * height:
            raise Exception("Invalid arguments")

        cmd = f"dim a9[{len(img)}]"

        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()

        written = self.stream.WriteFloats("a9", img)

        
        cmd = f"imgs(a9, {x}, {y}, {width}, {height}, {transform}, {scaleWidth}, {scaleHeight})"

        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        
        return res.success

    def DrawImage(self, img, x: int, y: int, width: int, height: int, transform: int) -> bool:
        return self.DrawImageScale(img, x, y, width, height, transform, 1, 1)


from DUELink.Analog import AnalogController
from DUELink.Button import ButtonController
from DUELink.Digital import DigitalController
from DUELink.Graphics import GraphicsController
from DUELink.GraphicsType import GraphicsTypeController
from DUELink.DistanceSensor import DistanceSensorController
from DUELink.Frequency import FrequencyController
from DUELink.I2C import I2cController
from DUELink.Infrared import InfraredController
from DUELink.System import SystemController
from DUELink.SerialInterface import SerialInterface
from DUELink.Servo import ServoController
from DUELink.Spi import SpiController
from DUELink.Engine import EngineController
from DUELink.DeviceConfiguration import DeviceConfiguration
from DUELink.Temperature import TemperatureController
from DUELink.Humidity import HudimityController
from DUELink.Sound import SoundController
from DUELink.Temperature import TemperatureSensorType
from DUELink.Humidity import HumiditySensorType
from DUELink.Stream import StreamController
from DUELink.DMX import DMXController
from DUELink.FileSystem import FileSystemController
from DUELink.Otp import OtpController
from DUELink.Pulse import PulseController
from DUELink.Rtc import RtcController
from DUELink.Uart import UartController

from enum import Enum
import platform
class DUELinkController:

    def __init__(self, comPort: str):
        if comPort is None:
            raise ValueError(f"Invalid comport: {comPort}")
        try:
            self.__Connect(comPort)
        except:
            raise Exception(f"Could not connect to the comport: {comPort}")
        
        if self.transport is None:
            raise Exception(f"transport is null")
        
        self.Stream = StreamController(self.transport)
        self.Analog = AnalogController(self.transport)
        self.Digital = DigitalController(self.transport)        
        self.Servo = ServoController(self.transport)
        self.Frequency = FrequencyController(self.transport)        
        self.Infrared = InfraredController(self.transport)
        self.Button = ButtonController(self.transport)
        self.Distance = DistanceSensorController(self.transport)                        
        self.Engine = EngineController(self.transport)
        self.Temperature = TemperatureController(self.transport)
        self.Humidity = HudimityController(self.transport)
        self.System = SystemController(self.transport)        
        self.GraphicsType = GraphicsTypeController()                
        self.TemperatureSensorType = TemperatureSensorType()
        self.HumiditySensorType = HumiditySensorType()       
        self.Pulse = PulseController(self.transport)        
        
        self.DMX = DMXController(self.transport,self.Stream)
        self.FileSystem = FileSystemController(self.transport,self.Stream)
        self.Otp = OtpController(self.transport,self.Stream)        
        self.Rtc = RtcController(self.transport,self.Stream)
        self.I2c = I2cController(self.transport,self.Stream)
        self.Spi = SpiController(self.transport,self.Stream)
        self.Uart = UartController(self.transport,self.Stream)
        self.Sound = SoundController(self.transport,self.Stream)
        self.Graphics = GraphicsController(self.transport,self.Stream)
        
        
    
    def __Connect(self, comPort: str):
        self.transport = SerialInterface(comPort)
        self.transport.Connect()

        # self.Version = self.transport.GetVersion()[1].strip()

        # if self.Version == "" or self.Version == "GHI Electronics DUELink v00.00:0000:00.09":
        #     raise Exception("The device is not supported.")
        
        self.DeviceConfig = DeviceConfiguration()

        # if self.Version[len(self.Version) -1] == 'P':
        #     self.DeviceConfig.IsPulse = True
        #     self.DeviceConfig.MaxPinIO = 23
        #     self.DeviceConfig.MaxPinAnalog = 29
        # elif self.Version[len(self.Version) -1] == 'I':
        #     self.DeviceConfig.IsPico = True
        #     self.DeviceConfig.MaxPinIO = 29
        #     self.DeviceConfig.MaxPinAnalog = 29  
        # elif self.Version[len(self.Version) -1] == 'F':
        #     self.DeviceConfig.IsFlea = True
        #     self.DeviceConfig.MaxPinIO = 11
        #     self.DeviceConfig.MaxPinAnalog = 29    
        # elif self.Version[len(self.Version) -1] == 'E':
        #     self.DeviceConfig.IsEdge = True
        #     self.DeviceConfig.MaxPinIO = 22
        #     self.DeviceConfig.MaxPinAnalog = 11  
        # elif self.Version[len(self.Version) -1] == 'R':
        #     self.DeviceConfig.IsRave = True
        #     self.DeviceConfig.MaxPinIO = 23
        #     self.DeviceConfig.MaxPinAnalog = 29
        # elif self.Version[len(self.Version) -1] == 'T':
        #     self.DeviceConfig.IsTick = True
        #     self.DeviceConfig.MaxPinIO = 23
        #     self.DeviceConfig.MaxPinAnalog = 11
        # elif self.Version[len(self.Version) -1] == 'D':
        #     self.DeviceConfig.IsDue = True
        #     self.DeviceConfig.MaxPinIO = 15
        #     self.DeviceConfig.MaxPinAnalog = 10

        self.transport.DeviceConfig = self.DeviceConfig        

    def Disconnect(self):
        self.transport.Disconnect()
    
    def Shutdown(self, pin: int):
        cmd = f'shtdn({pin})'
        self.transport.WriteCommand(cmd)
        response = self.transport.ReadResponse()
        return response.success    

    def GetConnectionPort():
        try:
            from serial.tools.list_ports import comports
        except ImportError:
            return ""
        
        if comports:
            com_ports_list = list(comports())
            ebb_ports_list = []
            for port in com_ports_list:               
                if port.vid ==0x1B9F and port.pid==0xF300:
                    if (platform.system() == 'Windows'):
                        return port.name                    
                    else:
                        return port.device

        return ""
    
    def __get_ReadTimeout(self):
        return self.transport.ReadTimeout

    def __set_ReadTimeout(self, value: int):
        self.transport.ReadTimeout = value 

    ReadTimeout = property(__get_ReadTimeout, __set_ReadTimeout)

    def __get_EnabledAsio(self):
        return self.transport.EnabledAsio

    def __set_EnabledAsio(self, value: int):
        self.transport.EnabledAsio = value 

    EnabledAsio = property(__get_EnabledAsio, __set_EnabledAsio)
   
         

        
        





from enum import Enum
from DUELink.SerialInterface import SerialInterface
from DUELink.Stream import StreamController

class DMXController:   

    def __init__(self, transport:SerialInterface, stream:StreamController):
        self.transport = transport
        self.stream = stream

    def Write(self, channel_data: bytes)->bool:
        count = len(channel_data)
        # declare b9 array
        cmd = f"dim b9[{count}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        # write data to b9
        ret = self.stream.WriteBytes("b9",channel_data)

        # write b9 to dmx
        self.transport.WriteCommand("DmxW(b9)")
        ret = self.transport.ReadResponse()

        return ret.success


    def Read(self, channel: int)->int:
        cmd = f"DmxR({channel})"
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()

        if ret.success:            
            try:
                value = int(ret.response)
                return value
            except:
                return -1

        return -1
    
    def Ready(self)->int:
        cmd = f"DmxRdy()"
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()

        if ret.success:            
            try:
                value = int(ret.response)
                return value
            except:
                return 0

        return 0
    
    def Update(self)->bool:
        cmd = f"DmxU()"
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()

        return ret.success
        




       






class InfraredController:
    def __init__(self, transport):
        self.transport = transport

    def Read(self)->int:
        cmd = "irread()"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()

        if res.success:
            try:
                return int(res.response)
            except:
                pass
        return -1
    
    def Write(self, command: int)->bool:
        cmd = f"IrWrite({command})"
        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()
        return res.success

    def Enable(self, txpin:int, rxpin: int)->bool:
        cmd = f"iren({txpin}, {rxpin})"
        self.transport.WriteCommand(cmd)

        res = self.transport.ReadResponse()
        return res.success



from enum import Enum
import time
from DUELink.SerialInterface import SerialInterface
import struct

class StreamController:   

    def __init__(self, transport:SerialInterface):
        self.transport = transport

    def WriteSpi(self, dataWrite: bytes):
        count = len(dataWrite)

        cmd = f"strmspi({count})"
        self.transport.WriteCommand(cmd)

        # wait for prompt &
        while self.transport.BytesToRead() == 0:
            time.sleep(0.001)
        
        prompt = self.transport.ReadByte()

        if prompt != '&':
            raise Exception("Invalid or no responses")
        
        # ready to write data
        self.transport.WriteRawData(dataWrite,0, count)

        # read x\r\n> (asio(1) not return this)
        ret = self.transport.ReadResponse()

        if ret.success:
            try:
                return int(ret.response)
            except:
                return 0                
        
        return 0

    def WriteBytes(self, arr: str, dataWrite: bytes):
        if dataWrite is None or dataWrite == 0:
            return 0

        count = len(dataWrite)

        # declare b1 array
        cmd = f"strmwr({arr},{count})"
        self.transport.WriteCommand(cmd)

        # wait for prompt &
        while self.transport.BytesToRead() == 0:
            time.sleep(0.001)
        
        prompt = self.transport.ReadByte()

        if prompt != '&':
            raise Exception("Invalid or no responses")
        
        # ready to write data
        self.transport.WriteRawData(dataWrite,0, count)

        # read x\r\n> (asio(1) not return this)
        ret = self.transport.ReadResponse()

        if ret.success:
            try:
                return int(ret.response)
            except:
                return 0                
        
        return 0
    
    def WriteFloats(self, arr: str, dataWrite: float):
        if dataWrite is None or dataWrite == 0:
            return 0
        count = len(dataWrite)

        # declare b1 array
        cmd = f"strmwr({arr},{count})"
        self.transport.WriteCommand(cmd)

        # wait for prompt &
        while self.transport.BytesToRead() == 0:
            time.sleep(0.001)
        
        prompt = self.transport.ReadByte()

        if prompt != '&':
            raise Exception("Invalid or no responses")
        
        # ready to write data
        for i in range (0, count):
            float_bytes = struct.pack('>f', dataWrite[i])
            float_bytes_lsb = float_bytes[::-1]
            self.transport.WriteRawData(float_bytes_lsb,0, 4)        

        # read x\r\n> (asio(1) not return this)
        ret = self.transport.ReadResponse()

        if ret.success:
            try:
                return int(ret.response)
            except:
                return 0                
        
        return 0
    
    def ReadBytes(self, arr: str, dataRead: bytes):
        if dataRead is None or dataRead == 0:
            return 0
        count = len(dataRead)

        # declare b1 array
        cmd = f"strmrd({arr},{count})"
        self.transport.WriteCommand(cmd)

        # wait for prompt &
        while self.transport.BytesToRead() == 0:
            time.sleep(0.001)
        
        prompt = self.transport.ReadByte()

        if prompt != '&':
            raise Exception("Invalid or no responses")
        
        # ready to read data
        self.transport.ReadRawData(dataRead,0, count)

        # read x\r\n> (asio(1) not return this)
        ret = self.transport.ReadResponse()

        if ret.success:
            try:
                return int(ret.response)
            except:
                return 0                
        
        return 0
    
    def ReadFloats(self, arr: str, dataRead: float):
        if dataRead is None or dataRead == 0:
            return 0
        count = len(dataRead)

        # declare b1 array
        cmd = f"strmrd({arr},{count})"
        self.transport.WriteCommand(cmd)

        # wait for prompt &
        while self.transport.BytesToRead() == 0:
            time.sleep(0.001)
        
        prompt = self.transport.ReadByte()

        if prompt != '&':
            raise Exception("Invalid or no responses")
        
        # ready to read data
        raw_bytes = bytearray(4)
        for i in range (0, count):
            self.transport.ReadRawData(raw_bytes,0, 4)
            raw_bytes_lsb = raw_bytes[::-1]
            dataRead[i] = struct.unpack('f', raw_bytes)[0]        

        # read x\r\n> (asio(1) not return this)
        ret = self.transport.ReadResponse()

        if ret.success:
            try:
                return int(ret.response)
            except:
                return 0                
        
        return 0



        





        
    
       



from DUELink.SerialInterface import SerialInterface

class AnalogController:
    def __init__(self, transport:SerialInterface):
        self.transport = transport
        self.Fixed_Frequency = 50

    def VoltRead(self, pin):

        if pin not in self.transport.DeviceConfig.AnalogPins:
            raise ValueError("Invalid pin. Enter a valid analog pin.")

        cmd = "vread({0})".format(pin)

        self.transport.WriteCommand(cmd)

        ret = self.transport.ReadResponse()

        if ret.success:
            try:
                return float(ret.response)
            except:
                pass

        return -1
    
    def Read(self, pin):

        if pin not in self.transport.DeviceConfig.AnalogPins:
            raise ValueError("Invalid pin. Enter a valid analog pin.")

        cmd = "aread({0})".format(pin)

        self.transport.WriteCommand(cmd)

        ret = self.transport.ReadResponse()

        if ret.success:
            try:
                return float(ret.response)
            except:
                pass

        return -1
    
    def Write(self, pin, duty_cycle):
        
        if pin not in self.transport.DeviceConfig.PWMPins: # Led
            raise ValueError('Invalid pin. Enter a valid pwm pin.')

        if duty_cycle < 0 or duty_cycle > 1:
            raise ValueError('Duty cycle must be in the range 0..1')

        cmd = f'awrite({pin}, {duty_cycle})'
        self.transport.WriteCommand(cmd)

        ret = self.transport.ReadResponse()

        return ret.success
    
    def ReadVcc(self):
        cmd = f"readvcc()"
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()

        if ret.success:
            try:
                return float(ret.response)
            except:
                pass

        return -1



class FrequencyController:
    def __init__(self, transport):
        self.transport = transport
        self.MaxFrequency = 24000000
        self.MinFrequency = 16

    def Write(self, pin: int, frequency, duration_ms=0, dutycyle=50)->bool:
        if frequency < self.MinFrequency or frequency > self.MaxFrequency:
            raise ValueError("Frequency must be in range 16Hz...24000000Hz")



        if dutycyle < 0 or dutycyle > 1:
            raise ValueError("dutycyle must be in range 0..100")
        
        if pin not in self.transport.DeviceConfig.PWMPins:
            raise ValueError("Invalid pin used for frequency")

        cmd = "freq({}, {}, {}, {})".format(pin, frequency, duration_ms, dutycyle)
        self.transport.WriteCommand(cmd)

        res = self.transport.ReadResponse()

        return res.success



from DUELink.SerialInterface import SerialInterface
from DUELink.Stream import StreamController

class UartController:
    def __init__(self, transport:SerialInterface, stream:StreamController):
        self.transport = transport
        self.stream = stream

    def Configuration(self, baudrate: int, rx_buf_size: int)->bool:
        cmd = "SerCfg({0}, {1})".format(baudrate, rx_buf_size)
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()
        return ret.success

    def WriteByte(self, data: int)->bool:
        cmd = "SerWr({})".format(data)
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()
        return ret.success        
    
    def WriteBytes(self, data: bytes)->int:
        count = len(data)
        cmd = f"dim b9[{count}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        written = self.stream.WriteBytes("b9", data)

        self.transport.WriteCommand("SerWrs(b9)")
        ret = self.transport.ReadResponse()

        if (ret.success):
            return written

        return 0
    
    def ReadByte(self):        
        self.transport.WriteCommand("SerRd()")
        res = self.transport.ReadResponse()
        if res.success:
            try:
                data = int(res.response)
                return data
            except:
                pass
        return 0
    
    def ReadBytes(self, data: bytearray, timeout_ms: int)->int:
        count = len(data)
        cmd = f"dim b9[{count}]"
        self.transport.WriteCommand(cmd)
        self.transport.ReadResponse()

        cmd = f"SerRds(b9, {timeout_ms})"
        self.transport.WriteCommand(cmd)
        ret = self.transport.ReadResponse()

        read = self.stream.ReadBytes("b9",data )

        if (ret.success):
            return read

        return 0

    def BytesToRead(self)->int:        
        self.transport.WriteCommand("SerB2R()")
        ret = self.transport.ReadResponse()
        if ret.success:
            try:
                ready = int(ret.response)
                return ready
            except:
                pass
        return 0
    
    def Discard(self)->bool:        
        self.transport.WriteCommand("SerDisc()")
        ret = self.transport.ReadResponse()
        
        return ret.success
    

    



import setuptools

setuptools.setup(
    name="duelink",
    version="1.2.4",
    author="GHI Electronics",
    author_email="support@ghielectronics.com",
    license="MIT",
    description="GHI Electronics DUELink Python library.",
    long_description=open("README.md", encoding="utf-8").read(),
    long_description_content_type="text/markdown",
    url="https://www.duelink.com/",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
    ],
    install_requires=[
        "pyserial"
    ],
    python_requires=">=3.6",
)



from enum import Enum

class ButtonController:   

    def __init__(self, transport):
        self.transport = transport
        
    def Enable(self, pin: int, state: bool) -> bool:         
        cmd = f"btnen({pin}, {int(state)})"

        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()

        return res.success
    
    def Down(self, pin: int) -> bool:

      
        cmd = f"btndown({pin})"

        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()

        if res.success:
            try:
                return int(res.response) == 1
            except:
                pass

        return False
    
    def Up(self, pin: int) -> bool:

        cmd = f"btnup({pin})"

        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()

        if res.success:
            try:
                return int(res.response) == 1
            except:
                pass

        return False  

    def Read(self, pin: int) -> bool:

        cmd = f"btnread({pin})"

        self.transport.WriteCommand(cmd)
        res = self.transport.ReadResponse()

        if res.success:
            try:
                return int(res.response) == 1
            except:
                pass

        return False         
       
